#ifndef SPI_DLL_H

#include "stdafx.h"
#include "windows.h"
#include "FTDI_Lib\ftd2xx.h"

#ifdef SPI_WRAPPER_EXPORTS
#define SPI_WRAPPER_API __declspec(dllexport)
#else
#define SPI_WRAPPER_API __declspec(dllimport)
#endif

#define MAX_CHAR_READ_IN_LOOP 2000
#define MAX_ARRAY_SIZE 1000

typedef struct stDevice_Info
{
	int Mode; //1 - 3 wire ; 2 - 4 wire
	int WriteModeCount;

	int AddressLen;
	int DataLen;
	int pinIndex_MISO[8];
	int MISOCnt;
	int pinIndex_MOSI[8];
	int MOSICnt;

	int RetainWPinValues[10];
	int RetainWPinCount;

	int RetainRPinValues[10];
	int RetainRPinCount;

	int Write_AddressPin[8];
	int Write_AddressPinCnt;
	int Write_AddressBitsPosArray[MAX_ARRAY_SIZE];
	int Write_AddressBitsPosArrayLen;

	int Write_DataPin[8];
	int Write_DataPinCnt;
	int Write_DataBitsPosArray[MAX_ARRAY_SIZE];
	int Write_DataBitsPosArrayLen;

	int WriteBytes[8][MAX_ARRAY_SIZE];
	int WriteBytesArrayLen;

	int Read_AddressPin[8];
	int Read_AddressPinCnt;
	int Read_AddressBitsPosArray[MAX_ARRAY_SIZE];
	int Read_AddressBitsPosArrayLen;

	int Read_DataPin[8];
	int Read_DataPinCnt;
	int Read_DataBitsPosArray[MAX_ARRAY_SIZE];
	int Read_DataBitsIndexArray[MAX_ARRAY_SIZE];
	int Read_DataBitsPosArrayLen;

	int ReadBytes[8][MAX_ARRAY_SIZE];
	int ReadBytesArrayLen;
	
}Device_Info;

Device_Info PortDeviceInfo[20][20];

int EVMCount=0;
FT_HANDLE PortHandle[30]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
char EVMDesc[50][50];
unsigned long EVMBaudRate[30];
unsigned char EVMSetIO[30];
int devCount[30];
int ConfigPin[30][8];

extern SPI_WRAPPER_API int nspi_wrapper;
FILE* fp;

//Custom Error Codes
#define EVM_INDEX_INVALID 5201
#define DEVICE_INDEX_INVALID 5202
#define FTDI_READ_FAILED 5203
#define INVALID_PATTERN_MODE 5204
#define IO_SETTING_MISMATCH 5205
#define PATTERN_COLUMN_COUNT_MISMATCH 5206
#define FORMAT_FILE_OPEN_ERROR 5207
#define INVALID_FORMAT_FILE 5208



#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int openUSBPort(char* devDesc,unsigned long baudRate,int* EVMIndex);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int initializeDevice(char* fileName,int EVMIndex, int* DeviceIndex);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int writeRegister(int EVMIndex, int devIndex, unsigned long int address, unsigned long int data);


#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int readRegister(int EVMIndex, int devIndex, unsigned long int address, unsigned long int* data);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int modifyBaudRate(int EVMIndex,unsigned long baudRate);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int closeAllPorts();

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int closeUSBPort(int EVMIndex);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int importFTHandle(FT_HANDLE ftHandleIn,unsigned long baudRate,int* EVMIndex);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int restartFTHandle (int EVMIndex);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int DLLVersion (float* DLLVersion);

#ifdef __cplusplus 
extern "C" 
#endif 
SPI_WRAPPER_API int exportFTHandle(int EVMIndex, FT_HANDLE* ftHandleOut);

#endif /*SPI_DLL_H*/
